##################################################
######                                  ##########
###### Algorithmen fuer Hochschultag    ##########
###### Graphitty 1.258 Demonstration    ##########
###### Christian Schindelhauer 31.10.97 ##########
###### Tobias Schoessler       04.11.97 ##########
###### Mathias Kolberg         04.11.97 ##########
##################################################

##################################
##### nuetzliche Hilfs- ##########
##### prozeduren        ##########
##################################




##################################################
# mark_selection
# ~~~~~~~~~~~~~~
# Frbt eine Selection mit einer Farbe ein
#
# s        : Selection, die eingefrbt werden soll
# color    : Farbe der Selection
#
proc mark_selection {s color} {
   for {set i 0} {$i < [$s count nodes]} {set i [expr $i +1]} {
       set n [$s node $i]
       set item [$n get node_attribute item]
       $n delete
       .c itemconfigure $item -fill $color
       }
   for {set i 0} {$i < [$s count edges]} {set i [expr $i +1]} {
       set n [$s edge $i]
       set item [$n get edge_attribute item]
       $n delete
       .c itemconfigure $item -fill $color
       }
}

##################################################
# exists_node_attribute {g n}
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~
# berprft, ob das Knotenattribut n im Graphen g
# existiert
#
# g       : Graph
# n       : Knotenattribut
#
proc exists_node_attribute {g n} {
   set e 0
   for {set i 0} {$i < [$g count node_attributes]} {set i [expr $i +1]} {
       set e [expr $e || ![string compare [$g node_attribute $i] $n]]
       }
   return $e
}

##################################################
# exists_edge_attribute {g n}
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~
# berprft, ob das Kantenattribut n im Graphen g
# existiert
#
# g       : Graph
# n       : Knotenattribut
#
proc exists_edge_attribute {g n} {
   set e 0
   for {set i 0} {$i < [$g count edge_attributes]} {set i [expr $i +1]} {
       set e [expr $e || ![string compare [$g edge_attribute $i] $n]]
       }
   return $e
}

##################################################
# ensure_node_attribute 
# ~~~~~~~~~~~~~~~~~~~~~
# Definiert im Graphen g ein neues Knotenattribut
# nur wenn noch kein gleichnamiges existiert
# (Initialisierungswert ist immer 0)
#
# g       : Graph
# name    : Knotenattribut
#
proc ensure_node_attribute {g name} {
  if {! [exists_node_attribute $g $name]} {
       $g new node_attribute $name 0
  }
}
    
##################################################
# ensure_edge_attribute
# ~~~~~~~~~~~~~~~~~~~~~ 
# Definiert im Graphen g ein neues Knotenattribut
# nur wenn noch kein gleichnamiges existiert
# (Initialisierungswert ist immer 0)
#
# g       : Graph
# name    : Knotenattribut
#
proc ensure_edge_attribute {g name} {
  if {! [exists_edge_attribute $g $name]} {
       $g new edge_attribute $name 0
  }
}      


##################################
###### Tiefensuche ###############
##################################


set dfs_c [$G new selection]
set dfs_v [$G new selection]
set dfs_m [$G new selection]

proc tiefen_suche {g current visited marked level} {
   if {[$current count nodes] > 1} {
         $current set node_attribute level $level
         $marked add $current
         set h [$current node 0]
         $current clear
         $current add $h
         $h delete
   } elseif {[$current count nodes] == 0} {
         if {[$marked count nodes] != 0} {
            $current clear
            set h [min_node_attribute $marked level]
            $current add $h
            $marked setminus $h
            $h delete
         }
   } else {
        $visited add $current
        $marked setminus $current
        $current select outs
        $visited add $current
        $current select targets
        
        $current setminus $visited
        $current setminus $marked
}
   mark_selection $current yellow
   mark_selection $visited red
   mark_selection $marked blue
   select_prim $visited
   select_sec  $marked
}




##################################
###### Breitensuche ##############
##################################


proc select_succ {g s} {
   set h [$g new selection]
   set old 0
   set new [expr [$s count nodes] + [$s count edges]] 
   while {$new != $old } {       
       set old $new
       breitensuche_intern $g $s
       set new [expr [$s count nodes]+ [$s count edges]]
       }
   $h add $s
   return $h
}

proc breitensuche_intern {g s} {
# 
   set n [$g new selection]
   $n add $s
   $n select targets
#
# n = Menge aller Zielknoten
#
   set e [$g new selection]
   $e add $s
   $e select outs
#
# d = Menge aller ausgehenden Kanten
#
   $s add $n
   $s add $e
#
#
   $n delete
   $e delete
#             Aufraeumen
# 
   return $s
   }


proc breitensuche {g s} {
# 
# Diese Prozedur ist notwendig um der primaeren Selektion
# eine neue Selektion zuzufuehren
#
   set h [$g new selection]
   breitensuche_intern $g $s
   $h add $s
   return $h
}


###################################
###  Dijkstra                ######
###################################


proc dijkstra {g start ziel} {
  set t [$g new selection]
  $t add all
  $t set node_attribute distance 999999
  $t intersect $start
  $t set node_attribute distance 0
  $t add all
  set a [$g new selection]
  while {[$t count nodes] != 0} {
       set u [min_node_attribute $t distance]
       set u_value [expr [$u get node_attribute distance] +1]
       $t setminus  $u
       $a clear
       $a add $u
       $a select outs
       $a select targets
       $a intersect $t
       for {set i 0} {$i < [$a count nodes]} {set i [expr $i +1]} {
           set n [$a node $i]
           set value [$n get node_attribute distance]
           if {$value > $u_value} {
                $n set node_attribute distance $u_value
                }
           $n delete
       }
       $u delete
   }
   $t delete
### $a existiert noch
   $a clear
### Ab hier wird der Wegegraph gezeichnet...
   set u [min_node_attribute $ziel distance]
   $a add $u
   set u_value [$u get node_attribute distance]
   if {$u_value != 999999} {
     mark_selection $u green
     while {$u_value != 0} {
        $u select ins
        $u select sources
        set u [min_node_attribute $u distance]
        $a add $u
        set u_value [$u get node_attribute distance]
     }
  $u delete
   select_prim $a
   mark_selection $a green
  }
  $a delete
}


###################################
### Graphenprodukt, das ###########
###################################


proc g_factor {g sl sr z} {
## g: Graph
## sl: linker Teilgraph
## sr: rechter Teilgarph
## z: Zoomfaktor fuer Darstellung
##
## Lokale Attribute und Selektionen
##
  $g new node_attribute L 0
  $g new node_attribute R 0
  set l_nodes [$sl count nodes]
  set r_nodes [$sr count nodes]
  set neu_nodes [$g new selection]
  set src [$g new selection]
  set tgt [$g new selection]
#
# Linker und Rechter Ursprungsteilgraph werden markiert
# in den Attributen L und R --- alle neuen Knoten werden
# erstellt
#
  for {set l 1} {$l <= $l_nodes} {set l [expr $l +1]} {
     for {set r 1} {$r <= $r_nodes} {set r [expr $r +1]} {
          set neu [$g new node]
          $neu_nodes add $neu
          $neu set node_attribute L $l
          $neu set node_attribute R $r
	  $sl set node_attr L $l [expr $l-1]
	  $sr set node_attr R $r [expr $r-1]
          $neu set node_attribute x [expr  \
              $z*([$sl get node_attr x [expr $l-1]]   \
              +   [$sr get node_attr x [expr $r-1]])]
          $neu set node_attribute y [expr \
              $z*([$sl get node_attr y [expr $l-1]]  \
              +   [$sr get node_attr y [expr $r-1]])]
      }}
#
# Die Kanten im linken Graph werden mit den Knoten im rechten
# Graph mulipliziert
#
   set l_edges [$sl count edges]
   set r_edges [$sr count edges]
   for {set l 1} {$l <= $l_edges} {set l [expr $l +1]} {
     set el [$sl edge [expr $l-1]]
     $el select sources
     set source_l [$el get node_attribute L]
     set el [$sl edge [expr $l-1]]
     $el select targets
     set target_l [$el get node_attribute L]
     for {set r 1} {$r <= $r_nodes} {set r [expr $r +1]} {
         $src add $neu_nodes
         $src select node_attribute L $source_l
         $src select node_attribute R $r
         $tgt add $neu_nodes
         $tgt select node_attribute L $target_l
         $tgt select node_attribute R $r
         $g new edge $src $tgt
   }}

#
# Wie eben nur anders herum
#
   for {set r 1} {$r <= $r_edges} {set r [expr $r +1]} {
     set er [$sr edge [expr $r-1]]
     $er select sources
     set source_r [$er get node_attribute R]
     set er [$sr edge [expr $r-1]]
     $er select targets
     set target_r [$er get node_attribute R]
     for {set l 1} {$l <= $l_nodes} {set l [expr $l +1]} {
         $src add $neu_nodes
         $src select node_attribute R $source_r
         $src select node_attribute L $l
         $tgt add $neu_nodes
         $tgt select node_attribute R $target_r
         $tgt select node_attribute L $l
         $g new edge $src $tgt
   }}
##
## Aufraeumen
##
$neu_nodes delete
$src delete
$tgt delete
$g delete node_attribute L
$g delete node_attribute R
rebuild
}

###########################################

proc kill_connected {g s} {
   set h [$g new selection]
   set k [$g new selection]
   while {[expr [$s count nodes] + [$s count edges]] > 0} {
       $k clear
       $h clear
       $k add $s
       $k select ins&outs
       $h add $k
       $k add $h
       $k add $s
       $k select targets
       $h add $k
       $k add $s
       $k add $h
       $k select sources
       $h add $k
       $s erase
       $s add $h 
    }
   }
 proc select_con {g s} {
   set h [$g new selection]
   set k [$g new selection]
   while {[expr [$s count nodes] + [$s count edges]] != \
          [expr [$h count nodes] + [$h count edges]] } {
       $s add $h
       $h add $s
       $k add $s
       $k select ins&outs
       $h add $k
       $k add $h
       $k select targets
       $h add $k
       $k add $h
       $k select sources
       $h add $k
    }
   return $h
   }

#############################################

### Baeume der Tiefe d und Spreizung x*5

proc tree {d x} {
   set root [new_node $x [expr 500 - $d * 50]]
   if {$d > 0} {
          set leftson  [tree [expr $d-1]  [expr $x - pow(2,$d) * 5]]
          set rightson [tree [expr $d-1]  [expr $x + pow(2,$d) * 5]]
          new_edge $root $leftson
          new_edge $root $rightson
          }
   return $root
}

############################################
set pi [expr acos(0) * 2]


proc circle_graph {no rad gr} {
    set shift 250
    set n0 [new_node  [expr $rad+$shift] $shift]
   set n $n0 
   for {set i 1} {$i < $no} {set i [expr $i +1]} {
            set n_alt $n
            set n  [new_node [expr ($rad * cos(2 * 3.14159 * $i / $no))+$shift]  \
                    [expr $rad * sin(2 * 3.14159 * $i / $no)+$shift]]
            new_edge $n_alt $n
            }
   new_edge $n $n0
   }

proc complete_graph {no rad gr} {
   set v [$gr new selection]
   for {set i 1} {$i <= $no} {set i [expr $i +1]} {
            $v add [new_node [expr $rad * cos(2 * 3.14159 * $i / $no)+250]  \
                            [expr $rad * sin(2 * 3.14159 * $i / $no)+250]]
            }
   $gr new edge $v $v
   set v0 [$gr new selection]
   set v1 [$gr new selection]
   for {set i 0} {$i <= [expr $no-1]} {set i [expr $i +1]} {
            $v0 clear
            $v1 clear
            $v0 add [$v node $i]
            $v1 add $v0
            $v0 select ins
            $v1 select outs
            $v1 & $v0
            $v1 erase
            }
    rebuild
    }       

proc center_point {gr cs} {
   set l [$gr new selection]
   set h [$gr new selection]
   $h add $cs
   $h select targets
   $l add $h
   $h add $cs 
   $h select sources
   $l add $h
   $l add $cs
   set no [$l count nodes]
   if {$no == 0} {
              return {0 0}
   } else {
       set x 0
       set y 0
       for {set i 0} {$i <= [expr $no-1]} {set i [expr $i +1]} {    
         set x [expr $x + [[$l node $i] get node_attribute x]] 
         set y [expr $y + [[$l node $i] get node_attribute y]] 
       }
     set x [expr $x/$no]
     set y [expr $y/$no]
     return [list $x $y]
   }
  }

proc sel_scale {gr cs ob z} {
    set x 0
    set y 0
    set c [center_point $gr $cs]
    set cx [lindex $c 0]
    set cy [lindex $c 1]
    set no [$ob count nodes]
    for {set i 0} {$i <= [expr $no-1]} {set i [expr $i +1]} {    
        set x [[$ob node $i] get node_attribute x]
        set y [[$ob node $i] get node_attribute y]
        set hx [expr (($x - $cx) * $z) + $cx]
        set hy [expr (($y - $cy) * $z) + $cy]
        [$ob node $i] set node_attribute x $hx
        [$ob node $i] set node_attribute y $hy
    }
    rebuild
 }       

proc sel_xy_scale {gr cs ob zx zy} {
    set x 0
    set y 0
    set c [center_point $gr $cs]
    set cx [lindex $c 0]
    set cy [lindex $c 1]
    set no [$ob count nodes]
    for {set i 0} {$i <= [expr $no-1]} {set i [expr $i +1]} {    
        set x [[$ob node $i] get node_attribute x]
        set y [[$ob node $i] get node_attribute y]
        set hx [expr (($x - $cx) * $zx) + $cx]
        set hy [expr (($y - $cy) * $zy) + $cy]
        [$ob node $i] set node_attribute x $hx
        [$ob node $i] set node_attribute y $hy
    }
    rebuild
 }       



proc labeln {g} {
    $G new node_attribute label 0
    set s [$g new selection]
    $s add all
    for {set i 1} {$i <= [$g count nodes]} {set i [expr $i +1]} {
         set s2 [$s node [expr $i-1]]
         $s2 set node_attribute label $i
         $s2 delete 
         }
         
}

proc pfeil {g} {
    set s [$g new selection]
    for {set i 1} {$i <= [$g count nodes]} {set i [expr $i +1]} {
         $s add all
         $s select node_attribute label $i
         $g new edge $s $s
         }
         
}


#######################
# Butterfly Graph

proc butterfly {d dx dy } {
# d = Tiefe (Kantentiefe)
# dx/dy = Abstaende der Knoten
    global G
    ensure_edge_attribute $G direction
    ensure_node_attribute $G index
    return [butterfly-rec $d 0 0 $dx 0 $dy]
    }

proc butterfly-rec {d indexoffset xoffset dx yoffset dy } {
# x/y-offset verschiebt den Butterfly um x,y
# indexoffset Position des des Butterfly
    global G
    set pow_d [expr 1 << $d]  
# pow_d = 2^d
    set b [$G new selection]
    if {$d > 0} {
        set e0 [$G new selection]
        set e [$G new selection]
	set l  [butterfly-rec [expr $d-1] $indexoffset $xoffset $dx [expr $yoffset+$dy] $dy]
	set r  [butterfly-rec [expr $d-1] [expr $indexoffset + $pow_d/2]     \
                                     [expr $xoffset+ $dx * $pow_d/2]  \
		                     $dx [expr $yoffset+$dy] $dy]
        for {set i 0} {$i < $pow_d} { set i [expr $i +1]} {
            set n [new_node [expr $xoffset + $dx * $i]   $yoffset]
            $b add $n
            $n set node_attribute index [expr $i + $indexoffset]
            set l_node [node_attribute_equals $l index [expr $indexoffset + [leftindex  $d $i]]]
            new_edge $n $l_node
            $e0 clear 
            $e clear
            $e add $n
            $e0 add $n
            $e0 select outs
	    set r_node [node_attribute_equals $r index [expr $indexoffset + [rightindex $d $i]]]
            new_edge $n $r_node
            $e select outs
            $e  set edge_attribute direction 1
            $e0 set edge_attribute direction 0
            $r_node delete
            $l_node delete
       }
       $e delete
       $e0 delete
       $l delete
       $r delete
    } else {
       set n [new_node $xoffset  $yoffset]
       $b add $n
       $n set node_attribute index $indexoffset
    }
  return $b
}
proc leftindex {d index} {
    set pow_d2 [expr 1 << [expr $d-1]]
    if {$index < $pow_d2}  {return  $index } else  {return [expr $index-$pow_d2]}
}
proc rightindex {d index} {
      set pow_d2 [expr 1 << [expr $d-1]]
    if {$index >= $pow_d2}  {return $index } else {return [expr $index+$pow_d2]}
}

proc left_right_node {v} {
    set n $new selection
    $n add $v
    $n select outs
    $n select targets
    if {$n count nodes != 2 } {error "ausserhalb des Butterfly-Graphen"}
    set n0 $n node 0
    set n1 $n node 1    
    if {[$n0 get node_attribute index] < [$n1 get node_attribute index]} {
         set r [list $n0 $n1]
       } else {
         set r [list $n1 $n0] 
     }
     $n delete
     return $r
}

proc get_input_node {butterfly index} {
     return [node_attribute_equals $butterfly index $index]
}
proc random_int {d} {
    set r 0
    for {set i 0} {$i < $d} {set i [expr $i+1]} {
	set r [expr ($r << 1) | [random_bit]]
    }
    return $r
}

ensure_node_attribute $G info
ensure_edge_attribute $G info

proc init_butter {b d} {
    global G    
    for {set i 0} {$i < (1 << $d)} { set i [expr $i +1]} {
        set n [$b node $i]
        $n set node_attribute info [random_int $d]
        $n delete
    }
    set level [$G new selection]
    $level add $b
    for {set i 0} {$i < $d} { set i [expr $i +1]} {
        $level select outs
        $level set edge_attribute info -1
        mark_selection $level black
        $level select targets
        $level set node_attribute info -1
        mark_selection $level white
    }
    $level delete
    mark_selection $b red    
}

proc edge-2-node {b d} {
    global G
    set level [$G new selection]
    $level add $b
    for {set i 0} {$i < $d} { set i [expr $i +1]} {
        $level select outs
        $level select targets
	for {set j 0} {$j < (1 << $d)} {set j [expr $j+1]} {
	    set n [$level node $j]
            set info [$n get node_attribute info]
            set index [$n get node_attribute index]
            if {$info == -1} {
                $n select ins 
                set e0 [$n edge 0]
                set e1 [$n edge 1]
                $e0 select sources
                $e1 select sources
                if {[$e0 get node_attribute index] == $index} {
                      set e0 [$n edge 0]
                      set e1 [$n edge 1]
                } else {
                      set e0 [$n edge 1]
                      set e1 [$n edge 0]
		}
                $n select targets
                if {[$e0 get edge_attribute info] != -1} {
		    $n set node_attribute info [$e0 get edge_attribute info]
                    mark_selection $n red
                    mark_selection $e0 green
                    $e0 set edge_attribute info -1
                    $e0 select sources 
                    mark_selection $e0 green
                    $e0 set node_attribute info -1
                } elseif { [$e1 get edge_attribute info] != -1} {
		    $n set node_attribute info [$e1 get edge_attribute info]
                    mark_selection $n red
                    mark_selection $e1 green
                    $e1 set edge_attribute info -1
                    $e1 select sources
                    mark_selection $e1 green
                    $e1 set node_attribute info -1
		}
                $e0 delete
                $e1 delete
	    }
            if {$i == [expr $d-1]} {
                if {[$n get node_attribute info] != -1} {
                  mark_selection $n blue
                  $n set node_attribute info -1
		}
            }
            $n delete
	}           
    }
}     

proc node-2-edge {b d} {
    global G
    set level [$G new selection]
    $level add $b
    for {set i 0} {$i < $d} { set i [expr $i +1]} {
	for {set j 0} {$j < (1 << $d)} {set j [expr $j+1]} {
	    set n [$level node $j]
            set info [$n get node_attribute info]
            if {$info != -1} {
		set drct [expr ($info & (1 << $i)) != 0]
                $n select outs 
                set e0 [$n edge 0]
                set e1 [$n edge 1]
                if {$drct == [$e0 get edge_attribute direction]} {
                  $e0 set edge_attribute info $info
                  mark_selection $e0 red                  
                } else {
                  $e1 set edge_attribute info $info
                  mark_selection $e1 red                 
		}
                $e0 delete
                $e1 delete
	    }
            $n delete
	}
        $level select outs
        $level select targets
    }
}     

set last 139677525
proc random_bit { } {
    global last
    set last [expr ((78040283 * $last) >> 2) + 200385759]
    return [expr $last & 1]
}
proc node_attribute_equals {s attr value} {
    global G
    set r [$G new selection]
    for {set i 0} {($i< [$s count nodes]) && ([$r count nodes] == 0)} {set i [expr $i +1]} {
 	 set n [$s node $i]
         if {[$n get node_attribute $attr] == $value} {
                 $r delete
                 set r $n
           } else { 
                 $n delete
           }
      }
      return $r
}
#########################
# Eulerscher Kantenzug

### __bfs
###   Tiefensuche (bzgl. Knoten) innerhalb eines Teilgraphen subGraph
###   mit Startknoten sel.
proc __bfs {subGraph sel} {
#  set _ins [$sel clone]
  set _outs [$sel clone]
  set old 0
  set new [$sel count nodes]
  
  while {$old != $new} {
    set old $new

#    $_ins select ins
#    $_ins & $subGraph
#    $_ins select sources
#    $_ins & $subGraph

    $_outs select outs
    $_outs & $subGraph
    $_outs select targets
    $_outs & $subGraph

#    $sel add $_ins
    $sel add $_outs
    set new [$sel count nodes]
  }
  $_outs delete
#  $_ins delete

  return $sel
}


### is_bridge
###   untersucht, ob die Kante e eine Bruecke innerhalb eines
###   Teilgraphen subGraph ist.
proc is_bridge {subGraph e} {
  set from [$e clone]   ;# Startknoten
  $from select sources
  set to [$e clone]     ;# Zielknoten
  $to select targets
  
  $subGraph - $e
  __bfs $subGraph $to
  $subGraph + $e
  $to & $from
  set count [$to count nodes]
  
  $to delete
  $from delete
  
  return [expr 1 - $count]
}


### euler_init
###
proc euler_init {} {
  global prim sec

  mark_selection $prim green
  $sec add all
}

### euler
###
proc euler {subGraph node} {
  global G

  if {[$subGraph count edges] == 0} {
    mark_selection $node green
    puts "Eulerscher Pfad gefunden!"
    return
  }
  
  set _outs [$node clone]
  $_outs select outs
  $_outs & $subGraph
  set n_edges [$_outs count edges]
  
  if {$n_edges == 0} {
    puts "Eulerscher Pfad nicht moeglich!"
    return
  }
  
  if {$n_edges >= 2} {
    for {set i 0} {$i < $n_edges} {incr i} {
      if {[is_bridge $subGraph [$_outs edge $i]] == 0} { break }  ;# Kante i ist keine Bruecke
    }
    if {$i == $n_edges} {
      puts "Eulerscher Pfad nicht moeglich!"
      return
    }
    $_outs & [$_outs edge $i]
  }

  mark_selection $_outs green
  $subGraph - $_outs
  $_outs select targets
 # damit prim aktualisiert wird, sind die folgenden beiden Zeilen noetig
  $node clear
  $node add $_outs
  
  $_outs delete
}



###########
# Alienmode

set mode normal
proc alien_button {} {
    global mode
    if { $mode=="normal" } {
	puts "Alien Mode ON"	
	set mode alien
	.balien configure -relief sunken
	.c bind node <Enter> { 
	    set obj [getobj current]
	    set x [$obj get node_attr x]
	    set y [$obj get node_attr y]
	    $obj set node_attr x [expr $x+5*($x-[X %x])]
	    $obj set node_attr y [expr $y+5*($y-[Y %y])]
	    rebuild
	}  
    } else {
	puts "Alien Mode OFF"
	set mode normal
	.balien configure -relief raised 
	.c bind node <Enter> { 
	    
	}
    }
}


###########
# Swirl

proc swirl { from to inc startnum startcirc } {
    global G
    for { set i $from } { $i<$to } {incr i $inc} {
	circle_graph [expr 20+$i] [expr 30+($i*10)] $G
    }
}
    

# Button ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#

# Tiefensuche
button .bdfsini   -text "Init Tiefensuche" -borderwidth $BorderWidth \
-background #FFCCCC -command {$dfs_c clear; $dfs_v clear; $dfs_m clear;\
$dfs_c add $prim; set t 0; ensure_node_attribute $G level}
pack .bdfsini -in .buttons -side top -fill x 

button .bdfs   -text "Tiefensuche" -borderwidth $BorderWidth \
-background #FFDDDD -command {set t [expr $t-1]; \
tiefen_suche $G $dfs_c $dfs_v $dfs_m $t}
pack .bdfs -in .buttons -side top -fill x 


# Breitensuche
button .breite   -text "Breitensuche" -borderwidth $BorderWidth -background #CCCCFF \
         -command {select_prim [breitensuche $G $prim]}
button .selsuc   -text "Successor" -borderwidth $BorderWidth -background #DDDDFF\
         -command {select_prim [select_succ $G $prim]}
pack .breite -in .buttons -side top -fill x
pack .selsuc -in .buttons -side top -fill x


# Min Path
button .bdijkstra   -text "min path" -borderwidth $BorderWidth \
	-background #CCFFCC -command {
    ensure_node_attribute $G distance
    dijkstra $G $prim $sec
}
pack .bdijkstra -in .buttons -side top -fill x 

# Graphprodukt
button .bfactor   -text "rot * blau" -borderwidth $BorderWidth \
-background #FFFFCC -command {g_factor $G $prim $sec 1; rebuild}
pack .bfactor -in .buttons -side top -fill x 

# Alien Mode
button .balien   -text "Alien Mode" -borderwidth $BorderWidth -background #CCFFFF -command { alien_button }
pack .balien -in .buttons -side top -fill x 


# Butterfly
button .bmakebutter   -text "Make Butterfly" -borderwidth $BorderWidth \
    -background #fca -command {set D 4; set BUTT [butterfly $D 50 50]}
button .bsortinit   -text "Init Sort" -borderwidth $BorderWidth \
    -background #fda -command {init_butter $BUTT $D; set SWITCH 0;}
button .bsortstep   -text "Sort Step" -borderwidth $BorderWidth \
    -background #fea -command {set SWITCH [expr ! $SWITCH];      \
				   if {$SWITCH} {node-2-edge $BUTT $D} else {edge-2-node $BUTT $D}}

pack .bmakebutter -in .buttons -side top -fill x 
pack .bsortinit -in .buttons -side top -fill x 
pack .bsortstep -in .buttons -side top -fill x 


# Eulerscher Kantenzug
button .euler_init   -text "Euler-Path Init" \
-borderwidth $BorderWidth \
-background #aaf \
-command {euler_init}
pack .euler_init -in .buttons -side top -fill x

button .euler   -text "Euler-Path Step" \
-borderwidth $BorderWidth \
-background #bbf \
-command {euler $sec $prim}
pack .euler -in .buttons -side top -fill x




# Menus ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

proc menu_complete_graph {} {
    global status
    global input1
    global input2
    global G
    get2strings .d "Complete Graph " "Number of Nodes:" "Radius:" \
	    OK CANCEL "5" "100"
    if {$status=="ok"} {
	complete_graph $input1 $input2 $G
	put "created ...!"
	return
    } else {
	put "canceled ...!"
    }
}

proc menu_tree {} {
    global status
    global input1
    global input2
    get2strings .d "Tree " "Depth:" "Start at x:" \
	    OK CANCEL "4" "200"
    if {$status=="ok"} {
	tree $input1 $input2
	put "created ...!"
	return
    } else {
	put "canceled ...!"
    }
}

proc menu_circle {} {
    global status
    global input1
    global input2
    global G
    get2strings .d "Circle Graph" "Numver of Nodes:" "Radius:" \
	    OK CANCEL "15" "100"
    if {$status=="ok"} {
	circle_graph $input1 $input2 $G
	put "created ...!"
	return
    } else {
	put "canceled ...!"
    }
}

proc menu_butter {} {
    global status
    global input1
    global input2
    getstring .d "Butterfly Graph" "Depth:" \
	    OK CANCEL "5"
    if {$status=="ok"} {
	uplevel {
	    set BUTT [butterfly $input1 50 50]
	    set D $input1
	}
	put "created ...!"
	return
    } else {
	put "canceled ...!"
    }
}



menubutton .menu.template -bd 1 -highlightthickness 0 -text "Template" -underline 0 -menu .menu.template.m
menu .menu.template.m -bd 1 -activeborderwidth 1
.menu.template.m add command -label "Complete" -underline 0 -command menu_complete_graph
.menu.template.m add command -label "Tree" -underline 0 -command menu_tree 
.menu.template.m add command -label "Circle" -underline 0 -command menu_circle
.menu.template.m add command -label "Butterfly" -underline 0 -command menu_butter


pack .menu.template -side left


